import { ChangeDetectionStrategy, ChangeDetectorRef, Component, inject, OnDestroy, OnInit } from '@angular/core';
import { ActivatedRoute, Router } from '@angular/router';
import {
    DowntimeObject,
    DowntimesServiceParams,
    DowntimesServiceRoot,
    getDefaultDowntimesServiceParamsParams
} from '../downtimes.interface';
import { Subscription } from 'rxjs';
import { AsyncPipe, formatDate, NgForOf, NgIf } from '@angular/common';
import {
    CardBodyComponent,
    CardComponent,
    CardFooterComponent,
    CardHeaderComponent,
    CardTitleDirective,
    ColComponent,
    ContainerComponent,
    FormCheckComponent,
    FormCheckInputDirective,
    FormCheckLabelDirective,
    FormControlDirective,
    FormDirective,
    InputGroupComponent,
    InputGroupTextDirective,
    ModalService,
    NavComponent,
    NavItemComponent,
    RowComponent,
    TableDirective
} from '@coreui/angular';
import { NotyService } from '../../../layouts/coreui/noty.service';
import { PaginatorChangeEvent } from '../../../layouts/coreui/paginator/paginator.interface';
import { MatSort, MatSortHeader, Sort } from '@angular/material/sort';
import { CancelAllItem } from '../cancel-hostdowntime-modal/cancel-hostdowntime.interface';
import { DELETE_SERVICE_TOKEN } from '../../../tokens/delete-injection.token';
import { DowntimesService } from '../downtimes.service';
import { PermissionsService } from '../../../permissions/permissions.service';
import { SelectionServiceService } from '../../../layouts/coreui/select-all/selection-service.service';
import { TranslocoDirective, TranslocoPipe, TranslocoService } from '@jsverse/transloco';
import { FaIconComponent } from '@fortawesome/angular-fontawesome';
import { XsButtonDirective } from '../../../layouts/coreui/xsbutton-directive/xsbutton.directive';
import { DebounceDirective } from '../../../directives/debounce.directive';
import { FormsModule } from '@angular/forms';
import { TableLoaderComponent } from '../../../layouts/primeng/loading/table-loader/table-loader.component';
import { NoRecordsComponent } from '../../../layouts/coreui/no-records/no-records.component';
import {
    PaginateOrScrollComponent
} from '../../../layouts/coreui/paginator/paginate-or-scroll/paginate-or-scroll.component';
import { ItemSelectComponent } from '../../../layouts/coreui/select-all/item-select/item-select.component';
import { PermissionDirective } from '../../../permissions/permission.directive';
import { SelectAllComponent } from '../../../layouts/coreui/select-all/select-all.component';
import { DowntimeSimpleIconComponent } from '../downtime-simple-icon/downtime-simple-icon.component';
import {
    ServiceBrowserMenuConfig,
    ServicesBrowserMenuComponent
} from '../../services/services-browser-menu/services-browser-menu.component';
import {
    CancelServicedowntimeModalComponent
} from '../cancel-servicedowntime-modal/cancel-servicedowntime-modal.component';

@Component({
    selector: 'oitc-downtimes-service',
    imports: [
        TranslocoDirective,
        CardComponent,
        CardHeaderComponent,
        CardTitleDirective,
        NgIf,
        NavComponent,
        NavItemComponent,
        FaIconComponent,
        XsButtonDirective,
        CardBodyComponent,
        ContainerComponent,
        RowComponent,
        ColComponent,
        FormDirective,
        InputGroupComponent,
        InputGroupTextDirective,
        DebounceDirective,
        FormControlDirective,
        TranslocoPipe,
        FormsModule,
        FormCheckComponent,
        FormCheckInputDirective,
        FormCheckLabelDirective,
        TableLoaderComponent,
        TableDirective,
        MatSort,
        MatSortHeader,
        NoRecordsComponent,
        PaginateOrScrollComponent,
        CardFooterComponent,
        AsyncPipe,
        ItemSelectComponent,
        NgForOf,
        PermissionDirective,
        SelectAllComponent,
        DowntimeSimpleIconComponent,
        CancelServicedowntimeModalComponent,
        ServicesBrowserMenuComponent
    ],
    templateUrl: './downtimes-service.component.html',
    styleUrl: './downtimes-service.component.css',
    providers: [
        {provide: DELETE_SERVICE_TOKEN, useClass: DowntimesService}, // Inject the DowntimesService into the CancelAllModalComponent
    ],
    changeDetection: ChangeDetectionStrategy.OnPush
})
export class DowntimesServiceComponent implements OnInit, OnDestroy {
    private serviceUuid: string = '';
    private DowntimesService = inject(DowntimesService);
    public readonly PermissionsService = inject(PermissionsService);
    public readonly route = inject(ActivatedRoute);
    public readonly router = inject(Router);

    public params: DowntimesServiceParams = getDefaultDowntimesServiceParamsParams();
    public was_cancelled: boolean = false;
    public was_not_cancelled: boolean = false;

    public serviceDowntimes?: DowntimesServiceRoot;
    public hideFilter: boolean = true;
    private subscriptions: Subscription = new Subscription();
    public from = formatDate(this.params['filter[from]'], 'yyyy-MM-ddTHH:mm', 'en-US');
    public to = formatDate(this.params['filter[to]'], 'yyyy-MM-ddTHH:mm', 'en-US');

    public lastUpdated: Date = new Date(); // Used to tell child components to reload data

    private cdr = inject(ChangeDetectorRef);

    public serviceBrowserConfig?: ServiceBrowserMenuConfig;
    public selectedItems: any[] = [];
    private SelectionServiceService: SelectionServiceService = inject(SelectionServiceService);
    private readonly modalService = inject(ModalService);
    private readonly notyService = inject(NotyService);
    private readonly TranslocoService: TranslocoService = inject(TranslocoService);


    public ngOnInit(): void {
        this.serviceUuid = String(this.route.snapshot.paramMap.get('uuid'));

        this.loadServicedowntimes();

        // Show the nice loader
        this.serviceBrowserConfig = {
            serviceUuid: this.serviceUuid,
            hostUuid: '', // will be added as soon as we have the data
            showReschedulingButton: false,
            showBackButton: true
        };
        this.cdr.markForCheck();
    }

    public ngOnDestroy(): void {
        this.subscriptions.unsubscribe();
    }

    public loadServicedowntimes() {
        this.params['filter[DowntimeServices.was_cancelled]'] = '';
        if (this.was_cancelled !== this.was_not_cancelled) {
            this.params['filter[DowntimeServices.was_cancelled]'] = 'false';
            if (this.was_cancelled) {
                this.params['filter[DowntimeServices.was_cancelled]'] = 'true';
            }
        }

        this.params['filter[DowntimeServices.service_description]'] = this.serviceUuid;
        this.params['filter[from]'] = formatDate(new Date(this.from), 'dd.MM.y HH:mm', 'en-US');
        this.params['filter[to]'] = formatDate(new Date(this.to), 'dd.MM.y HH:mm', 'en-US');


        this.subscriptions.add(this.DowntimesService.getDowntimesService(this.params)
            .subscribe((result) => {
                this.serviceDowntimes = result;

                // Define the configuration for the ServiceBrowserMenuComponent because we know the hostUuid now
                this.serviceBrowserConfig = {
                    serviceUuid: this.serviceUuid,
                    hostUuid: result.hostUuid,
                    showReschedulingButton: false,
                    showBackButton: true
                };


                // Tell the menu to update
                this.lastUpdated = new Date();

                this.cdr.markForCheck();
            })
        );
    }

    // Show or hide the filter
    public toggleFilter() {
        this.hideFilter = !this.hideFilter;
    }

    public resetFilter() {
        this.params = getDefaultDowntimesServiceParamsParams();
        this.from = formatDate(this.params['filter[from]'], 'yyyy-MM-ddTHH:mm', 'en-US');
        this.to = formatDate(this.params['filter[to]'], 'yyyy-MM-ddTHH:mm', 'en-US');

        this.loadServicedowntimes();
        this.cdr.markForCheck();
    }

    // Callback for Paginator or Scroll Index Component
    public onPaginatorChange(change: PaginatorChangeEvent): void {
        this.params.page = change.page;
        this.params.scroll = change.scroll;
        this.loadServicedowntimes();
    }


    // Callback when a filter has changed
    public onFilterChange(event: Event) {
        this.params.page = 1;
        this.loadServicedowntimes();
    }

    // Callback when sort has changed
    public onSortChange(sort: Sort) {
        if (sort.direction) {
            this.params.sort = sort.active;
            this.params.direction = sort.direction;
            this.loadServicedowntimes();
        }
    }


    // Generic callback whenever a mass action (like delete all) has been finished
    public onMassActionComplete(success: boolean) {
        if (success) {
            this.loadServicedowntimes();
        }
    }

    // Open the Delete All Modal
    public toggleCancelDowntimeModal(hostDowntime?: DowntimeObject) {
        let items: CancelAllItem[] = [];
        if (hostDowntime) {
            // User just want to delete a single command

            items = [{
                id: hostDowntime.internal_downtime_id
            }];


        } else {
            // User clicked on delete selected button
            items = this.SelectionServiceService.getSelectedItems().map((item): CancelAllItem => {
                return {
                    id: item.internal_downtime_id
                };
            });
        }

        if (items.length === 0) {
            const message = this.TranslocoService.translate('No items selected!');
            this.notyService.genericError(message);
            return;
        }

        // Pass selection to the modal
        this.selectedItems = items;
        this.cdr.markForCheck();

        // open modal
        this.modalService.toggle({
            show: true,
            id: 'cancelServiceAllModal',
        });
    }
}
