import {
    ChangeDetectionStrategy,
    ChangeDetectorRef,
    Component,
    effect,
    inject,
    input,
    OnDestroy,
    OnInit,
    output
} from '@angular/core';
import { ColComponent, RowComponent, TooltipDirective } from '@coreui/angular';
import { FaIconComponent } from '@fortawesome/angular-fontawesome';
import { Subscription } from 'rxjs';
import { HostsService } from '../hosts.service';
import { TranslocoDirective, TranslocoPipe, TranslocoService } from '@jsverse/transloco';
import { AsyncPipe, NgClass, NgIf, TitleCasePipe } from '@angular/common';
import { SkeletonModule } from 'primeng/skeleton';
import { PermissionsService } from '../../../permissions/permissions.service';
import { XsButtonDirective } from '../../../layouts/coreui/xsbutton-directive/xsbutton.directive';
import { RouterLink } from '@angular/router';
import { HoststatusIconComponent } from '../hoststatus-icon/hoststatus-icon.component';
import {
    BrowserMenuLoaderComponent
} from '../../../layouts/primeng/loading/browser-menu-loader/browser-menu-loader.component';
import { HostsBrowserRoot } from '../hosts.interface';
import { HostStatusNamePipe } from '../../../pipes/host-status-name.pipe';


export interface HostBrowserMenuConfig {
    hostUuid: string,
    showReschedulingButton?: boolean,
    rescheduleCallback?: () => void,
    showBackButton?: boolean
}

@Component({
    selector: 'oitc-hosts-browser-menu',
    imports: [
        ColComponent,
        RowComponent,
        FaIconComponent,
        NgIf,
        NgClass,
        TranslocoDirective,
        SkeletonModule,
        XsButtonDirective,
        RouterLink,
        TooltipDirective,
        TranslocoPipe,
        TitleCasePipe,
        HoststatusIconComponent,
        BrowserMenuLoaderComponent,
        AsyncPipe,
        HostStatusNamePipe
    ],
    templateUrl: './hosts-browser-menu.component.html',
    styleUrl: './hosts-browser-menu.component.css',
    changeDetection: ChangeDetectionStrategy.OnPush
})
export class HostsBrowserMenuComponent implements OnInit, OnDestroy {

    public config = input.required<HostBrowserMenuConfig>();
    public lastUpdated = input<Date>(); // Change the date to trigger an update from an external component

    public toggleRescheduling = output<boolean>();

    public data?: HostsBrowserRoot;
    public hostStatusTextClass: string = '';
    public isLoading = true;

    private subscriptions: Subscription = new Subscription();
    private readonly HostsService = inject(HostsService);
    public readonly PermissionsService: PermissionsService = inject(PermissionsService)
    private readonly TranslocoService: TranslocoService = inject(TranslocoService)
    private cdr = inject(ChangeDetectorRef);

    constructor() {
        effect(() => {
            this.loadData();
        });
    }

    public ngOnInit() {
        const lastUpdated = this.lastUpdated();
        if (!lastUpdated) {
            // If lastUpdate is undefined, load the current status data.
            // If lastUpdate is set to a date time, the OnChange Method will be triggered and do the loading instead.
            // This is to avoid duplicate loading of data.
            this.loadData();
        }
    }

    public ngOnDestroy() {
        this.subscriptions.unsubscribe();
    }


    public toggleReschedulingOutput() {
        this.toggleRescheduling.emit(true);
    }

    public loadData() {
        this.subscriptions.add(this.HostsService.getHostBrowser(this.config().hostUuid)
            .subscribe((result) => {
                this.cdr.markForCheck();
                this.isLoading = false;
                this.data = result;

                switch (this.data.hoststatus.current_state) {
                    case 0:
                        this.hostStatusTextClass = 'up';
                        break;
                    case 1:
                        this.hostStatusTextClass = 'down';
                        break;

                    default:
                        this.hostStatusTextClass = 'unreachable';
                }

            })
        );
    }

}
