import { ChangeDetectionStrategy, ChangeDetectorRef, Component, inject, OnDestroy, OnInit } from '@angular/core';
import {
    CardBodyComponent,
    CardComponent,
    CardFooterComponent,
    CardHeaderComponent,
    CardTitleDirective,
    ColComponent,
    ContainerComponent,
    DropdownComponent,
    DropdownItemDirective,
    DropdownMenuDirective,
    DropdownToggleDirective,
    FormCheckComponent,
    FormCheckInputDirective,
    FormCheckLabelDirective,
    FormControlDirective,
    FormDirective,
    InputGroupComponent,
    InputGroupTextDirective,
    ModalService,
    NavComponent,
    NavItemComponent,
    RowComponent,
    TableDirective,
    TooltipDirective
} from '@coreui/angular';
import { ActivatedRoute, Router, RouterLink } from '@angular/router';
import { PermissionDirective } from '../../../permissions/permission.directive';
import { FaIconComponent } from '@fortawesome/angular-fontawesome';
import { XsButtonDirective } from '../../../layouts/coreui/xsbutton-directive/xsbutton.directive';
import { IndexPage } from '../../../pages.interface';
import { HostsService } from '../hosts.service';
import { PermissionsService } from '../../../permissions/permissions.service';
import { Subscription } from 'rxjs';
import { NotyService } from '../../../layouts/coreui/noty.service';
import { MatSort, MatSortHeader, Sort } from '@angular/material/sort';
import { PaginatorChangeEvent } from '../../../layouts/coreui/paginator/paginator.interface';
import {
    getDefaultHostsIndexParams,
    getHostCurrentStateForApi,
    HostsCurrentStateFilter,
    HostsIndexParams,
    HostsIndexRoot
} from '../hosts.interface';
import { SelectionServiceService } from '../../../layouts/coreui/select-all/selection-service.service';
import { TranslocoDirective, TranslocoPipe, TranslocoService } from '@jsverse/transloco';
import { ExternalCommandsEnum } from '../../../enums/external-commands.enum';
import { HostAcknowledgeItem, HostDowntimeItem } from '../../../services/external-commands.service';
import { DebounceDirective } from '../../../directives/debounce.directive';
import { FormsModule } from '@angular/forms';
import { TrueFalseDirective } from '../../../directives/true-false.directive';
import { TableLoaderComponent } from '../../../layouts/primeng/loading/table-loader/table-loader.component';
import { AsyncPipe, NgIf } from '@angular/common';
import { ItemSelectComponent } from '../../../layouts/coreui/select-all/item-select/item-select.component';
import { HoststatusIconComponent } from '../hoststatus-icon/hoststatus-icon.component';
import {
    AcknowledgementIconComponent
} from '../../acknowledgements/acknowledgement-icon/acknowledgement-icon.component';
import { DowntimeIconComponent } from '../../downtimes/downtime-icon/downtime-icon.component';
import { TrustAsHtmlPipe } from '../../../pipes/trust-as-html.pipe';
import {
    PaginateOrScrollComponent
} from '../../../layouts/coreui/paginator/paginate-or-scroll/paginate-or-scroll.component';
import { NoRecordsComponent } from '../../../layouts/coreui/no-records/no-records.component';
import { SelectAllComponent } from '../../../layouts/coreui/select-all/select-all.component';
import {
    HostsMaintenanceModalComponent
} from '../../../components/hosts/hosts-maintenance-modal/hosts-maintenance-modal.component';
import {
    ServiceResetChecktimeModalComponent
} from '../../../components/services/service-reset-checktime-modal/service-reset-checktime-modal.component';
import {
    HostAcknowledgeModalComponent
} from '../../../components/hosts/host-acknowledge-modal/host-acknowledge-modal.component';

@Component({
    selector: 'oitc-hosts-index',
    imports: [
        CardFooterComponent,
        RouterLink,
        PermissionDirective,
        FaIconComponent,
        CardComponent,
        CardHeaderComponent,
        CardTitleDirective,
        NavComponent,
        NavItemComponent,
        XsButtonDirective,
        CardBodyComponent,
        ContainerComponent,
        RowComponent,
        ColComponent,
        FormDirective,
        InputGroupComponent,
        InputGroupTextDirective,
        DebounceDirective,
        TranslocoPipe,
        FormsModule,
        FormControlDirective,
        FormCheckComponent,
        FormCheckInputDirective,
        TrueFalseDirective,
        FormCheckLabelDirective,
        TranslocoDirective,
        TableLoaderComponent,
        TableDirective,
        MatSort,
        MatSortHeader,
        NgIf,
        TooltipDirective,
        ItemSelectComponent,
        AsyncPipe,
        HoststatusIconComponent,
        AcknowledgementIconComponent,
        DowntimeIconComponent,
        TrustAsHtmlPipe,
        PaginateOrScrollComponent,
        NoRecordsComponent,
        SelectAllComponent,
        DropdownComponent,
        DropdownToggleDirective,
        DropdownMenuDirective,
        DropdownItemDirective,
        HostsMaintenanceModalComponent,
        ServiceResetChecktimeModalComponent,
        HostAcknowledgeModalComponent
    ],
    templateUrl: './hosts-index.component.html',
    styleUrl: './hosts-index.component.css',
    changeDetection: ChangeDetectionStrategy.OnPush
})
export class HostsIndexComponent implements OnInit, OnDestroy, IndexPage {
    public params: HostsIndexParams = getDefaultHostsIndexParams();

    public currentStateFilter: HostsCurrentStateFilter = {
        up: false,
        down: false,
        unreachable: false
    };
    public state_typesFilter = {
        soft: false,
        hard: false
    };
    public acknowledgementsFilter = {
        acknowledged: false,
        not_acknowledged: false
    };
    public downtimeFilter = {
        in_downtime: false,
        not_in_downtime: false
    };

    public hosts?: HostsIndexRoot;
    public hideFilter: boolean = true;

    public selectedItems: any[] = [];

    public userFullname: string = '';

    private readonly HostsService = inject(HostsService);
    private subscriptions: Subscription = new Subscription();
    public readonly PermissionsService = inject(PermissionsService);
    public readonly route = inject(ActivatedRoute);
    public readonly router = inject(Router);
    private readonly TranslocoService: TranslocoService = inject(TranslocoService)
    private readonly notyService: NotyService = inject(NotyService);
    private SelectionServiceService: SelectionServiceService = inject(SelectionServiceService);
    private readonly modalService = inject(ModalService);

    private cdr = inject(ChangeDetectorRef);

    public ngOnInit() {

        this.route.queryParams.subscribe(params => {
            let hostname = params['hostname'] || undefined;
            if (hostname) {
                this.params['filter[HostMapping.display_name]'] = hostname;
            }

            let direction = params['direction'];
            if (direction && direction === 'asc' || direction === 'desc') {
                this.params.direction = direction;
            }

            let sort = params['sort'];
            if (sort) {
                this.params.sort = sort;
            }

            let inDowntime = params['inDowntime'] || undefined;
            if (inDowntime) {
                let isFilterInDowntime: boolean = inDowntime === 'true';
                this.downtimeFilter.in_downtime = isFilterInDowntime;
                this.downtimeFilter.not_in_downtime = !isFilterInDowntime;
            }

            let acknowledged = params['acknowledged'] || undefined;
            if (acknowledged) {
                let isFilterAcknowledged : boolean = acknowledged === 'true';
                this.acknowledgementsFilter.acknowledged = isFilterAcknowledged;
                this.acknowledgementsFilter.not_acknowledged = !isFilterAcknowledged;
            }

            let hoststate = params['hoststate'] || undefined;
            if (hoststate) {
                //first reset all states
                this.currentStateFilter.up = false;
                this.currentStateFilter.down = false;
                this.currentStateFilter.unreachable = false
                hoststate = [].concat(hoststate); // make sure we always get an array
                hoststate.forEach((state: any) => {
                    switch (parseInt(state, 10)) {
                        case 0:
                            this.currentStateFilter.up = true;
                            break;
                        case 1:
                            this.currentStateFilter.down = true;
                            break;
                        case 2:
                            this.currentStateFilter.unreachable = true;
                            break;
                        default:
                            break;
                    }
                });
            }

            // Process all query params first and then trigger the load function
            this.loadHosts();
        });

    }

    public ngOnDestroy() {
        this.subscriptions.unsubscribe();
    }

    public loadHosts() {
        this.SelectionServiceService.deselectAll();

        let hasBeenAcknowledged = '';
        let inDowntime = '';
        if (this.acknowledgementsFilter.acknowledged !== this.acknowledgementsFilter.not_acknowledged) {
            hasBeenAcknowledged = String(this.acknowledgementsFilter.acknowledged === true);
        }
        if (this.downtimeFilter.in_downtime !== this.downtimeFilter.not_in_downtime) {
            inDowntime = String(this.downtimeFilter.in_downtime === true);
        }

        let state_type: string = '';
        if (this.state_typesFilter.soft !== this.state_typesFilter.hard) {
            state_type = '0';
            if (this.state_typesFilter.hard) {
                state_type = '1';
            }
        }


        this.params['filter[Hoststatus.problem_has_been_acknowledged]'] = hasBeenAcknowledged;
        this.params['filter[Hoststatus.scheduled_downtime_depth]'] = inDowntime;
        this.params['filter[Hoststatus.is_hardstate]'] = state_type;
        this.params['filter[current_state][]'] = getHostCurrentStateForApi(this.currentStateFilter);

        this.subscriptions.add(this.HostsService.getIndex(this.params)
            .subscribe((result) => {
                this.hosts = result;
                this.userFullname = result.username;
                this.cdr.markForCheck();
            })
        );
    }

    // Show or hide the filter
    // Called by (click) - no manual change detection required
    public toggleFilter() {
        this.hideFilter = !this.hideFilter;
    }

    // Called by (click) - no manual change detection required
    public problemsOnly() {
        this.resetFilter(false);

        this.currentStateFilter = {
            up: false,
            down: true,
            unreachable: true
        };
        this.acknowledgementsFilter = {
            acknowledged: false,
            not_acknowledged: true
        };
        this.downtimeFilter = {
            in_downtime: false,
            not_in_downtime: true
        };

        this.cdr.markForCheck();

        this.loadHosts();

    }

    // Callback when a filter has changed
    public onFilterChange(event: any) {
        this.params.page = 1;
        this.loadHosts();
    }

    // Callback when sort has changed
    public onSortChange(sort: Sort) {
        if (sort.direction) {
            this.params.sort = sort.active;
            this.params.direction = sort.direction;
            this.loadHosts();
        }
    }

    // Callback for Paginator or Scroll Index Component
    public onPaginatorChange(change: PaginatorChangeEvent): void {
        this.params.page = change.page;
        this.params.scroll = change.scroll;
        this.loadHosts();
    }

    public resetFilter(doReload: boolean = true) {
        // Reset query parameters
        this.router.navigate([], {
            queryParams: {},
            queryParamsHandling: "",
        });

        this.params = getDefaultHostsIndexParams();
        this.currentStateFilter = {
            up: false,
            down: false,
            unreachable: false
        };
        this.state_typesFilter = {
            soft: false,
            hard: false
        };
        this.acknowledgementsFilter = {
            acknowledged: false,
            not_acknowledged: false
        };
        this.downtimeFilter = {
            in_downtime: false,
            not_in_downtime: false
        };

        if (doReload) {
            this.loadHosts();
        }
    }


    // Generic callback whenever a mass action (like delete all) has been finished
    public onMassActionComplete(success: boolean) {
        if (success) {
            this.loadHosts();
        }
    }

    public resetChecktime() {
        this.selectedItems = [];
        let selectedHostUuids: string[] = this.SelectionServiceService.getSelectedItems().map((item) => item.hostname);

        this.hosts?.hoststatus.forEach((item) => {
            if (typeof (item.hostname) === "undefined") {
                return;
            }
            if (!selectedHostUuids.includes(item.hostname)) {
                return;
            }

            this.selectedItems.push({
                command: ExternalCommandsEnum.rescheduleHost,
                hostUuid: item.hostname,
                type: 'hostOnly'
            });
        });

        if (this.selectedItems.length === 0) {
            const message = this.TranslocoService.translate('No items selected!');
            this.notyService.genericError(message);
            return;
        }
        this.modalService.toggle({
            show: true,
            id: 'serviceResetChecktimeModal'
        });
    }

    public toggleDowntimeModal() {
        let items: HostDowntimeItem[] = [];
        items = this.SelectionServiceService.getSelectedItems().map((item): HostDowntimeItem => {
            return {
                command: ExternalCommandsEnum.submitHostDowntime,
                hostUuid: item.hostname,
                start: 0,
                end: 0,
                author: this.userFullname,
                comment: '',
                downtimetype: ''
            };
        });

        this.selectedItems = items;
        if (items.length === 0) {
            const message = this.TranslocoService.translate('No items selected!');
            this.notyService.genericError(message);
            return;
        }
        this.modalService.toggle({
            show: true,
            id: 'hostMaintenanceModal',
        });
    }

    public acknowledgeStatus() {
        let items: HostAcknowledgeItem[] = [];
        items = this.SelectionServiceService.getSelectedItems().map((item): HostAcknowledgeItem => {
            return {
                command: ExternalCommandsEnum.submitHoststateAck,
                hostUuid: item.hostname,
                hostAckType: item.type,
                author: this.userFullname,
                comment: '',
                notify: true,
                sticky: false
            };
        });
        this.selectedItems = items;
        if (items.length === 0) {
            const message = this.TranslocoService.translate('No items selected!');
            this.notyService.genericError(message);
            return;
        }
        this.modalService.toggle({
            show: true,
            id: 'hostAcknowledgeModal',
        });
    }


    protected readonly String = String;
}
