import {
    ChangeDetectionStrategy,
    ChangeDetectorRef,
    Component,
    effect,
    inject,
    input,
    OnDestroy,
    OnInit,
    output
} from '@angular/core';
import { Subscription } from 'rxjs';
import { HostsService } from '../../hosts/hosts.service';
import { ServicesService } from '../services.service';
import { PermissionsService } from '../../../permissions/permissions.service';
import { TranslocoDirective, TranslocoPipe, TranslocoService } from '@jsverse/transloco';
import { ServicesBrowserRoot } from '../services.interface';
import { ColComponent, RowComponent, TooltipDirective } from '@coreui/angular';
import {
    BrowserMenuLoaderComponent
} from '../../../layouts/primeng/loading/browser-menu-loader/browser-menu-loader.component';
import { FaIconComponent } from '@fortawesome/angular-fontawesome';

import { AsyncPipe, NgClass, NgIf, TitleCasePipe } from '@angular/common';
import { XsButtonDirective } from '../../../layouts/coreui/xsbutton-directive/xsbutton.directive';
import {
    ServicestatusIconComponent
} from '../../../components/services/servicestatus-icon/servicestatus-icon.component';
import { RouterLink } from '@angular/router';
import { HostsBrowserRoot } from '../../hosts/hosts.interface';
import { ServiceStatusNamePipe } from '../../../pipes/service-status-name.pipe';


export interface ServiceBrowserMenuConfig {
    hostUuid: string,
    serviceUuid: string,
    showReschedulingButton?: boolean,
    rescheduleCallback?: () => void,
    showBackButton?: boolean
}


@Component({
    selector: 'oitc-services-browser-menu',
    imports: [
        BrowserMenuLoaderComponent,
        ColComponent,
        FaIconComponent,
        NgIf,
        RowComponent,
        TitleCasePipe,
        TranslocoDirective,
        TranslocoPipe,
        XsButtonDirective,
        ServicestatusIconComponent,
        NgClass,
        RouterLink,
        TooltipDirective,
        AsyncPipe,
        ServiceStatusNamePipe
    ],
    templateUrl: './services-browser-menu.component.html',
    styleUrl: './services-browser-menu.component.css',
    changeDetection: ChangeDetectionStrategy.OnPush
})
export class ServicesBrowserMenuComponent implements OnInit, OnDestroy {


    public config = input.required<ServiceBrowserMenuConfig>();
    public lastUpdated = input<Date>(); // Change the date to trigger an update from an external component

    public toggleRescheduling = output<boolean>();

    public data?: ServicesBrowserRoot;
    public hoststatus?: HostsBrowserRoot;
    public serviceStatusTextClass: string = '';
    public hostStatusTextClass: string = '';
    public isLoading = true;

    private subscriptions: Subscription = new Subscription();
    private readonly HostsService = inject(HostsService);
    private readonly ServicesService = inject(ServicesService);
    public readonly PermissionsService: PermissionsService = inject(PermissionsService)
    private readonly TranslocoService: TranslocoService = inject(TranslocoService)
    private cdr = inject(ChangeDetectorRef);

    constructor() {
        effect(() => {
            this.loadData();
        });
    }

    public ngOnInit() {
        const lastUpdated = this.lastUpdated();
        if (!lastUpdated) {
            // If lastUpdate is undefined, load the current status data.
            // If lastUpdate is set to a date time, the OnChange Method will be triggered and do the loading instead.
            // This is to avoid duplicate loading of data.
            this.loadData();
        }
    }

    public ngOnDestroy() {
        this.subscriptions.unsubscribe();
    }


    public toggleReschedulingOutput() {
        this.toggleRescheduling.emit(true);
    }

    public loadData() {
        if (this.config().hostUuid) {
            this.subscriptions.add(this.ServicesService.getServiceBrowser(this.config().serviceUuid)
                .subscribe((result) => {
                    this.cdr.markForCheck();

                    this.isLoading = false;
                    this.data = result;

                    switch (this.data.servicestatus.current_state) {
                        case 0:
                            this.serviceStatusTextClass = 'ok';
                            break;
                        case 1:
                            this.serviceStatusTextClass = 'warning';
                            break;
                        case 2:
                            this.serviceStatusTextClass = 'critical';
                            break;

                        default:
                            this.serviceStatusTextClass = 'unknown';
                    }
                })
            );

            this.subscriptions.add(this.HostsService.getHostBrowser(this.config().hostUuid)
                .subscribe((result) => {
                    this.cdr.markForCheck();
                    this.isLoading = false;
                    this.hoststatus = result;

                    switch (this.hoststatus.hoststatus.current_state) {
                        case 0:
                            this.hostStatusTextClass = 'up';
                            break;
                        case 1:
                            this.hostStatusTextClass = 'down';
                            break;

                        default:
                            this.hostStatusTextClass = 'unreachable';
                    }
                })
            );
        }
    }

}
