import { ChangeDetectionStrategy, ChangeDetectorRef, Component, inject, OnDestroy, OnInit } from '@angular/core';
import {
    CardBodyComponent,
    CardComponent, CardFooterComponent,
    CardHeaderComponent, CardTitleDirective,
    ColComponent,
    ContainerComponent, DropdownDividerDirective,
    FormControlDirective,
    FormDirective,
    InputGroupComponent,
    InputGroupTextDirective,
    ModalService,
    NavComponent,
    NavItemComponent,
    RowComponent, TableDirective
} from '@coreui/angular';
import { SelectionServiceService } from '../../../layouts/coreui/select-all/selection-service.service';
import { NotyService } from '../../../layouts/coreui/noty.service';
import { UsergroupsService } from '../usergroups.service';
import { ActivatedRoute, Router, RouterLink } from '@angular/router';
import {
    getDefaultUsergroupsIndexParams,
    Usergroup,
    UsergroupsIndexParams,
    UsergroupsIndexRoot
} from '../usergroups.interface';
import { DeleteAllItem } from '../../../layouts/coreui/delete-all-modal/delete-all.interface';
import { Subscription } from 'rxjs';
import { TranslocoDirective, TranslocoPipe, TranslocoService } from '@jsverse/transloco';
import { DELETE_SERVICE_TOKEN } from '../../../tokens/delete-injection.token';
import { IndexPage } from '../../../pages.interface';
import { PaginatorChangeEvent } from '../../../layouts/coreui/paginator/paginator.interface';
import { MatSort, MatSortHeader, Sort } from '@angular/material/sort';
import { FaIconComponent } from '@fortawesome/angular-fontawesome';
import { PermissionDirective } from '../../../permissions/permission.directive';
import { XsButtonDirective } from '../../../layouts/coreui/xsbutton-directive/xsbutton.directive';
import { DebounceDirective } from '../../../directives/debounce.directive';
import { FormsModule } from '@angular/forms';
import { TableLoaderComponent } from '../../../layouts/primeng/loading/table-loader/table-loader.component';
import { NgForOf, NgIf } from '@angular/common';
import { ItemSelectComponent } from '../../../layouts/coreui/select-all/item-select/item-select.component';
import { ActionsButtonComponent } from '../../../components/actions-button/actions-button.component';
import {
    ActionsButtonElementComponent
} from '../../../components/actions-button-element/actions-button-element.component';
import { NoRecordsComponent } from '../../../layouts/coreui/no-records/no-records.component';
import { SelectAllComponent } from '../../../layouts/coreui/select-all/select-all.component';
import {
    PaginateOrScrollComponent
} from '../../../layouts/coreui/paginator/paginate-or-scroll/paginate-or-scroll.component';
import { DeleteAllModalComponent } from '../../../layouts/coreui/delete-all-modal/delete-all-modal.component';


@Component({
    selector: 'oitc-usergroups-index',
    imports: [
        TranslocoDirective,
        RouterLink,
        FaIconComponent,
        PermissionDirective,
        CardComponent,
        CardHeaderComponent,
        NavComponent,
        NavItemComponent,
        XsButtonDirective,
        CardBodyComponent,
        ContainerComponent,
        RowComponent,
        ColComponent,
        FormDirective,
        InputGroupComponent,
        InputGroupTextDirective,
        DebounceDirective,
        FormsModule,
        TranslocoPipe,
        FormControlDirective,
        TableLoaderComponent,
        NgIf,
        TableDirective,
        MatSort,
        MatSortHeader,
        ItemSelectComponent,
        ActionsButtonComponent,
        ActionsButtonElementComponent,
        DropdownDividerDirective,
        NoRecordsComponent,
        SelectAllComponent,
        PaginateOrScrollComponent,
        CardFooterComponent,
        DeleteAllModalComponent,
        NgForOf,
        CardTitleDirective
    ],
    templateUrl: './usergroups-index.component.html',
    styleUrl: './usergroups-index.component.css',
    providers: [
        {provide: DELETE_SERVICE_TOKEN, useClass: UsergroupsService} // Inject the UsergroupsService into the DeleteAllModalComponent
    ],
    changeDetection: ChangeDetectionStrategy.OnPush
})
export class UsergroupsIndexComponent implements OnInit, OnDestroy, IndexPage {
    private readonly modalService: ModalService = inject(ModalService);
    private readonly SelectionServiceService: SelectionServiceService = inject(SelectionServiceService);
    private readonly TranslocoService = inject(TranslocoService);
    private readonly notyService: NotyService = inject(NotyService);
    private readonly subscriptions: Subscription = new Subscription();
    private readonly UsergroupsService: UsergroupsService = inject(UsergroupsService);
    private readonly route: ActivatedRoute = inject(ActivatedRoute);
    private readonly router: Router = inject(Router);
    private readonly cdr = inject(ChangeDetectorRef);

    protected params: UsergroupsIndexParams = getDefaultUsergroupsIndexParams();
    protected usergroups?: UsergroupsIndexRoot;
    protected selectedItems: DeleteAllItem[] = [];
    protected hideFilter: boolean = true;

    public ngOnInit(): void {
        this.subscriptions.add(this.route.queryParams.subscribe(params => {
            this.loadUsergroups();
        }));
    }

    public ngOnDestroy() {
        this.subscriptions.unsubscribe();
    }

    protected loadUsergroups() {
        this.SelectionServiceService.deselectAll();
        this.subscriptions.add(this.UsergroupsService.getIndex(this.params)
            .subscribe((result: UsergroupsIndexRoot) => {
                this.usergroups = result;
                this.cdr.markForCheck();
            }));
    }

    // Show or hide the filter
    public toggleFilter() {
        this.hideFilter = !this.hideFilter;
    }

    // Callback when a filter has changed
    public onFilterChange(event: any) {
        this.params.page = 1;
        this.loadUsergroups();
    }

    // Callback when sort has changed
    public onSortChange(sort: Sort) {
        if (sort.direction) {
            this.params.sort = sort.active;
            this.params.direction = sort.direction;
            this.loadUsergroups();
        }
    }

    // Callback for Paginator or Scroll Index Component
    public onPaginatorChange(change: PaginatorChangeEvent): void {
        this.params.page = change.page;
        this.params.scroll = change.scroll;
        this.loadUsergroups();
    }

    // Generic callback whenever a mass action (like delete all) has been finished
    public onMassActionComplete(success: boolean) {
        if (success) {
            this.loadUsergroups();
        }
    }

    public resetFilter() {
        this.params = getDefaultUsergroupsIndexParams();
        this.loadUsergroups();
    }

    // Open the Delete All Modal
    public toggleDeleteAllModal(usergroup?: Usergroup) {
        let items: DeleteAllItem[] = [];

        if (usergroup) {
            // User just want to delete a single command
            items = [{
                id: Number(usergroup.id),
                displayName: String(usergroup.name)
            }];
        } else {
            // User clicked on delete selected button
            items = this.SelectionServiceService.getSelectedItems().map((item): DeleteAllItem => {
                return {
                    id: item.id,
                    displayName: item.name
                };
            });
        }


        if (items.length === 0) {
            const message = this.TranslocoService.translate('No items selected!');
            this.notyService.genericError(message);
            return;
        }

        // Pass selection to the modal
        this.selectedItems = items;
        // open modal
        this.modalService.toggle({
            show: true,
            id: 'deleteAllModal',
        });
    }
}
