import { ChangeDetectionStrategy, ChangeDetectorRef, Component, inject, OnDestroy, OnInit } from '@angular/core';
import { getDefaultUsersIndexParams, User, UsersIndexParams, UsersIndexRoot } from '../users.interface';
import { SelectKeyValue } from '../../../layouts/primeng/select.interface';
import { Subscription } from 'rxjs';
import { UsersService } from '../users.service';
import { NotyService } from '../../../layouts/coreui/noty.service';
import { IndexPage } from '../../../pages.interface';
import { TranslocoDirective, TranslocoPipe, TranslocoService } from '@jsverse/transloco';
import { ActivatedRoute, Router, RouterLink } from '@angular/router';
import { PaginatorChangeEvent } from '../../../layouts/coreui/paginator/paginator.interface';
import { MatSort, MatSortHeader, Sort } from '@angular/material/sort';
import { PermissionDirective } from '../../../permissions/permission.directive';
import { FaIconComponent } from '@fortawesome/angular-fontawesome';
import {
    CardBodyComponent,
    CardComponent,
    CardFooterComponent,
    CardHeaderComponent,
    CardTitleDirective,
    ColComponent,
    ContainerComponent,
    DropdownDividerDirective,
    FormControlDirective,
    FormDirective,
    InputGroupComponent,
    InputGroupTextDirective,
    ModalService,
    NavComponent,
    NavItemComponent,
    RowComponent,
    TableDirective
} from '@coreui/angular';
import { XsButtonDirective } from '../../../layouts/coreui/xsbutton-directive/xsbutton.directive';
import { DebounceDirective } from '../../../directives/debounce.directive';
import { FormsModule } from '@angular/forms';
import { MultiSelectComponent } from '../../../layouts/primeng/multi-select/multi-select/multi-select.component';
import { TableLoaderComponent } from '../../../layouts/primeng/loading/table-loader/table-loader.component';
import { AsyncPipe, NgForOf, NgIf } from '@angular/common';
import { ItemSelectComponent } from '../../../layouts/coreui/select-all/item-select/item-select.component';
import { ActionsButtonComponent } from '../../../components/actions-button/actions-button.component';
import {
    ActionsButtonElementComponent
} from '../../../components/actions-button-element/actions-button-element.component';
import { DeleteAllItem } from '../../../layouts/coreui/delete-all-modal/delete-all.interface';
import { DELETE_SERVICE_TOKEN } from '../../../tokens/delete-injection.token';
import { SelectionServiceService } from '../../../layouts/coreui/select-all/selection-service.service';
import { PermissionsService } from '../../../permissions/permissions.service';
import { NoRecordsComponent } from '../../../layouts/coreui/no-records/no-records.component';
import { SelectAllComponent } from '../../../layouts/coreui/select-all/select-all.component';
import {
    PaginateOrScrollComponent
} from '../../../layouts/coreui/paginator/paginate-or-scroll/paginate-or-scroll.component';
import { DeleteAllModalComponent } from '../../../layouts/coreui/delete-all-modal/delete-all-modal.component';
import { UsergroupsService } from '../../usergroups/usergroups.service';

@Component({
    selector: 'oitc-users-index',
    imports: [
        TranslocoDirective,
        RouterLink,
        PermissionDirective,
        FaIconComponent,
        CardComponent,
        NavComponent,
        NavItemComponent,
        CardHeaderComponent,
        CardTitleDirective,
        XsButtonDirective,
        CardBodyComponent,
        ContainerComponent,
        RowComponent,
        ColComponent,
        FormDirective,
        InputGroupComponent,
        InputGroupTextDirective,
        DebounceDirective,
        FormsModule,
        FormControlDirective,
        TranslocoPipe,
        MultiSelectComponent,
        TableLoaderComponent,
        NgIf,
        TableDirective,
        MatSort,
        MatSortHeader,
        NgForOf,
        ItemSelectComponent,
        ActionsButtonComponent,
        ActionsButtonElementComponent,
        DropdownDividerDirective,
        AsyncPipe,
        NoRecordsComponent,
        SelectAllComponent,
        PaginateOrScrollComponent,
        CardFooterComponent,
        DeleteAllModalComponent
    ],
    templateUrl: './users-index.component.html',
    styleUrl: './users-index.component.css',
    providers: [
        {provide: DELETE_SERVICE_TOKEN, useClass: UsersService} // Inject the UsersService into the DeleteAllModalComponent
    ],
    changeDetection: ChangeDetectionStrategy.OnPush
})
export class UsersIndexComponent implements OnInit, OnDestroy, IndexPage {

    public params: UsersIndexParams = getDefaultUsersIndexParams();
    public users?: UsersIndexRoot;
    public usergroups: SelectKeyValue[] = [];
    public hideFilter: boolean = true;
    public selectedItems: DeleteAllItem[] = [];
    private subscriptions: Subscription = new Subscription();
    public readonly PermissionsService = inject(PermissionsService);
    private readonly UsersService = inject(UsersService);
    private readonly UsergroupsService = inject(UsergroupsService);
    private readonly SelectionServiceService = inject(SelectionServiceService);
    private readonly TranslocoService: TranslocoService = inject(TranslocoService)
    private readonly notyService: NotyService = inject(NotyService);
    private readonly modalService = inject(ModalService);
    private readonly route = inject(ActivatedRoute);
    private readonly router = inject(Router);
    private cdr = inject(ChangeDetectorRef);

    public ngOnInit(): void {
        this.subscriptions.add(this.route.queryParams.subscribe(params => {
            // Here, params is an object containing the current query parameters.
            // You can do something with these parameters here.
            //console.log(params);
            let usergroupId = params['usergroup_id'] || params['usergroup_id'];
            if (usergroupId) {
                usergroupId = Number(usergroupId);
                this.params['filter[Users.usergroup_id][]'] = [].concat(usergroupId); // make sure we always get an array
            }

            this.loadUsers();
            this.loadUsergroups();
        }));
    }

    public ngOnDestroy(): void {
        this.subscriptions.unsubscribe();
    }

    public loadUsers(): void {
        this.SelectionServiceService.deselectAll();
        this.subscriptions.add(
            this.UsersService.getIndex(this.params).subscribe((users) => {
                this.users = users;
                this.cdr.markForCheck();
            })
        );
    }

    public loadUsergroups(): void {
        this.subscriptions.add(
            this.UsergroupsService.loadUsergroups().subscribe((usergroups) => {
                this.usergroups = usergroups;
                this.cdr.markForCheck();
            })
        );
    }

    // Show or hide the filter
    public toggleFilter() {
        this.hideFilter = !this.hideFilter;
    }

    // Callback when a filter has changed
    public onFilterChange(event: any) {
        this.params.page = 1;
        this.loadUsers();
    }

    // Callback when sort has changed
    public onSortChange(sort: Sort) {
        if (sort.direction) {
            this.params.sort = sort.active;
            this.params.direction = sort.direction;
            this.loadUsers();
        }
    }

    // Callback for Paginator or Scroll Index Component
    public onPaginatorChange(change: PaginatorChangeEvent): void {
        this.params.page = change.page;
        this.params.scroll = change.scroll;
        this.loadUsers();
    }

    // Generic callback whenever a mass action (like delete all) has been finished
    public onMassActionComplete(success: boolean) {
        if (success) {
            this.loadUsers();
        }
    }

    public resetFilter() {
        this.params = getDefaultUsersIndexParams();
        this.loadUsers();
    }

    // Open the Delete All Modal
    public toggleDeleteAllModal(user?: User) {
        let items: DeleteAllItem[] = [];

        if (user) {
            // User just want to delete a single command
            items = [{
                id: Number(user.id),
                displayName: String(user.username)
            }];
        } else {
            // User clicked on delete selected button
            items = this.SelectionServiceService.getSelectedItems().map((item): DeleteAllItem => {
                return {
                    id: item.id,
                    displayName: item.username
                };
            });
        }


        if (items.length === 0) {
            const message = this.TranslocoService.translate('No items selected!');
            this.notyService.genericError(message);
            return;
        }

        // Pass selection to the modal
        this.selectedItems = items;
        // open modal
        this.modalService.toggle({
            show: true,
            id: 'deleteAllModal',
        });
    }
}
